@@ -7,7 +7,6 @@
 #' distribution). Must be strictly positive, need not be integer.
 #' @param prob probability of success on each trial
 #' @param mu alternative parametrization via mean
-#' @importFrom stats rnbinom
 #' @rdname rtrunc
 #' @export
 rtruncnbinom <- rtrunc.nbinom <- function(n, size, prob, mu, a = 0, b = Inf) {
@@ -15,26 +14,19 @@ rtruncnbinom <- rtrunc.nbinom <- function(n, size, prob, mu, a = 0, b = Inf) {
   sampleFromTruncated(mget(ls()))
 }
 
-#' @importFrom stats dnbinom pnbinom
+#' @rdname dtrunc
+#' @param ... size
 #' @export
-dtrunc.trunc_nbinom <- function(y, eta, a = 0, b = Inf, ...) {
+dtrunc.trunc_nbinom <- function(
+  y, size, prob, eta, a = 0, b = Inf, ...
+) {
+  if (missing(eta)) {
+    eta <- parameters2natural.parms_nbinom(c("size" = size, "prob" = prob))
+  }
   nsize <- attr(y, "parameters")$size
   proba <- attr(y, "parameters")$prob
-  my.dnbinom <- function(y, nsize, proba) dnbinom(y, size = nsize, prob = proba)
-  my.pnbinom <- function(z, nsize, proba) pnbinom(z, size = nsize, prob = proba)
-  dens <- ifelse((y < a) | (y > b), 0, my.dnbinom(y, nsize, proba))
-
-  if (!missing(a)) {
-    F.a <- my.pnbinom(a - 1, nsize, proba)
-  } else {
-    F.a <- 0
-  }
-  if (!missing(b)) {
-    F.b <- my.pnbinom(b, nsize, proba)
-  } else {
-    F.b <- 1
-  }
-  return(dens / (F.b - F.a))
+  dens <- rescaledDensities(y, a, b, dnbinom, pnbinom, nsize, proba)
+  return(dens)
 }
 
 #' @rdname dtrunc
@@ -43,10 +35,18 @@ dtrunc.trunc_nbinom <- function(y, eta, a = 0, b = Inf, ...) {
 dtruncnbinom <- dtrunc.trunc_nbinom
 
 #' @export
-init.parms.trunc_nbinom <- function(y) {
+#' @rdname dtrunc
+dtruncnbinom <- dtrunc.trunc_nbinom
+
+#' @export
+empiricalParameters.trunc_nbinom <- function(y, r, k, ...) {
   # Returns empirical parameter estimate for lambda
-  parms <- c("mean" = mean(y))
-  class(parms) <- "trunc_nbinom"
+  if (missing(r) || missing(k)) {
+    parms <- c("mean" = mean(y))
+  } else {
+    parms <- c("size" = r, "prob" = (r - 1) / (r + k - 1))
+  }
+  class(parms) <- "parms_nbinom"
   return(parms)
 }
 
@@ -54,12 +54,8 @@ sufficientT.trunc_nbinom <- function(y) {
   return(suff.T = y)
 }
 
-averageT.trunc_nbinom <- function(y) {
-  return(mean(y))
-}
-
 #' @export
-natural2parameters.trunc_nbinom <- function(eta) {
+natural2parameters.parms_nbinom <- function(eta, ...) {
   # eta: The natural parameters in a negative binomial distribution
   p <- c(mean = exp(eta))
   class(p) <- class(eta)
@@ -67,15 +63,19 @@ natural2parameters.trunc_nbinom <- function(eta) {
 }
 
 #' @export
-parameters2natural.trunc_nbinom <- function(parms) {
+parameters2natural.parms_nbinom <- function(parms, ...) {
   # parms: The p parameter in a negative binomial distribution
   # returns the natural parameters
-  eta <- log(parms)
-  class(eta) <- class(parms)
+  if (all(names(parms) == c("size", "prob"))) {
+    mean <- parms[["size"]] * (1 - parms[["prob"]]) / parms[["prob"]]
+  } else {
+    mean <- parms[["mean"]]
+  }
+  eta <- prepEta(log(mean), class(parms))
   return(eta)
 }
 
-getGradETinv.trunc_nbinom <- function(eta, r, ...) {
+getGradETinv.parms_nbinom <- function(eta, r = 1e3, ...) {
   # eta: Natural parameter
   # return the inverse of E.T differentiated with respect to eta
   p <- exp(eta)
