@@ -12,21 +12,13 @@ rtruncnorm <- rtrunc.normal <- function(n, mean, sd, a = -Inf, b = Inf) {
 }
 
 #' @export
-dtrunc.trunc_normal <- function(y, eta, a = -Inf, b = Inf) {
-  parm <- natural2parameters.trunc_normal(eta)
-  dens <- ifelse((y < a) | (y > b), 0, dnorm(y, mean = parm[1], sd = parm[2]))
-  if (!missing(a)) {
-    F.a <- pnorm(a, parm[1], parm[2])
-  } else {
-    F.a <- 0
+dtrunc.trunc_normal <- function(y, mean = 0, sd = 1, eta, a = -Inf, b = Inf, ...) {
+  if (missing(eta)) {
+    eta <- parameters2natural.parms_normal(c("mean" = mean, "sd" = sd))
   }
-  if (!missing(b)) {
-    F.b <- pnorm(b, parm[1], parm[2])
-  } else {
-    F.b <- 1
-  }
-  const <- 1 / (F.b - F.a)
-  return(dens * const)
+  parm <- natural2parameters.parms_normal(eta)
+  dens <- rescaledDensities(y, a, b, dnorm, pnorm, parm[1], parm[2])
+  return(dens)
 }
 
 #' @rdname dtrunc
@@ -34,10 +26,10 @@ dtrunc.trunc_normal <- function(y, eta, a = -Inf, b = Inf) {
 dtruncnorm <- dtrunc.trunc_normal
 
 #' @export
-init.parms.trunc_normal <- function(y) {
+empiricalParameters.trunc_normal <- function(y, ...) {
   # Returns empirical parameter estimates mean and sd
   parms <- c(mean = mean(y), sd = sqrt(var(y)))
-  class(parms) <- "trunc_normal"
+  class(parms) <- "parms_normal"
   return(parms)
 }
 
@@ -45,24 +37,21 @@ sufficientT.trunc_normal <- function(y) {
   return(suff.T = cbind(y, y^2))
 }
 
-averageT.trunc_normal <- function(y) {
-  return(apply(sufficientT.trunc_normal(y), 2, mean))
-}
-
 #' @export
-natural2parameters.trunc_normal <- function(eta) {
+natural2parameters.parms_normal <- function(eta, ...) {
   # eta: The natural parameters in a normal distribution
   # returns (mean,sigma)
-  parms <- c(mean = -0.5 * eta[1] / eta[2], sd = sqrt(-0.5 / eta[2]))
+  if (length(eta) != 2) stop("Eta must be a vector of two elements")
+  parms <- c("mean" = -0.5 * eta[[1]] / eta[[2]], "sd" = sqrt(-0.5 / eta[[2]]))
   class(parms) <- class(eta)
   return(parms)
 }
 
 #' @export
-parameters2natural.trunc_normal <- function(parms) {
+parameters2natural.parms_normal <- function(parms, ...) {
   # parms: The parameters mean and sd in a normal distribution
   # returns the natural parameters
-  eta <- c(eta.1 = parms[1], eta.2 = -0.5) / parms[2]^2
+  eta <- c(eta1 = parms[["mean"]], eta2 = -0.5) / parms[["sd"]]^2
   class(eta) <- class(parms)
   return(eta)
 }
@@ -77,8 +66,15 @@ getYseq.trunc_normal <- function(y, y.min, y.max, n = 100) {
   return(out)
 }
 
-getGradETinv.trunc_normal <- function(eta, ...) {
+getGradETinv.parms_normal <- function(eta, ...) {
   # eta: Natural parameter
   # return the inverse of E.T differentiated with respect to eta' : p x p matrix
-  return(A = solve(0.5 * matrix(c(-1 / eta[2], eta[1] / eta[2]^2, eta[1] / eta[2]^2, 1 / eta[2]^2 - eta[1]^2 / eta[2]^3), ncol = 2)))
+  A_inv <- 0.5 * matrix(
+    c(
+      -1 / eta[2], eta[1] / eta[2]^2,
+      eta[1] / eta[2]^2, 1 / eta[2]^2 - eta[1]^2 / eta[2]^3
+    ),
+    ncol = 2
+  )
+  return(A = solve(A_inv))
 }
